"""Translate the queue names to reference namespace by id, remove the namespace column.

Revision ID: 2430f55c41d5
Revises: 17f11e265e13
Create Date: 2014-09-30 17:31:33.308490

"""

# revision identifiers, used by Alembic.
revision = '2fb36d4be80d'
down_revision = '17f11e265e13'

from alembic import op
import sqlalchemy as sa

import re
from app import app
from data.database import QueueItem, User, db


NAMESPACE_EXTRACTOR = re.compile(r'^([a-z]+/)([a-z0-9_]+)(/.*$)')


def upgrade(tables):
    # Rename the namespace component of the existing queue items to reference user ids
    with app.config['DB_TRANSACTION_FACTORY'](db):
        for item in QueueItem.select():
            namespace_match = NAMESPACE_EXTRACTOR.match(item.queue_name)
            if namespace_match is not None:
                namespace_name = namespace_match.group(2)
                namespace_user = User.get(User.username == namespace_name)
                item.queue_name = '%s%s%s' % (namespace_match.group(1), str(namespace_user.id),
                                              namespace_match.group(3))
                item.save()
            else:
                raise RuntimeError('Invalid queue name: %s' % item.queue_name)

    op.create_index('repository_namespace_user_id', 'repository', ['namespace_user_id'], unique=False)
    op.drop_column('repository', 'namespace')


def downgrade(tables):
    # Add the namespace column back in and fill it in
    op.add_column('repository', sa.Column('namespace', sa.String(length=255)))
    op.drop_index('repository_namespace_user_id', table_name='repository')

    # Rename the namespace component of existing queue items to reference namespace strings
    with app.config['DB_TRANSACTION_FACTORY'](db):
        for item in QueueItem.select():
            namespace_match = NAMESPACE_EXTRACTOR.match(item.queue_name)
            if namespace_match is not None:
                namespace_id = namespace_match.group(2)
                namespace_user = User.get(User.id == namespace_id)
                item.queue_name = '%s%s%s' % (namespace_match.group(1),
                                              str(namespace_user.username),
                                              namespace_match.group(3))
                item.save()
            else:
                raise RuntimeError('Invalid queue name: %s' % item.queue_name)
